@@ -446,6 +446,7 @@ impl Einsum {
     }
 
     #[staticmethod]
+    #[args(nodes = "*", name = "None", out_axes_permutation)]
     pub fn new_outer_product(
         nodes: Vec<CircuitRc>,
         name: Option<Name>,
@@ -1418,7 +1419,7 @@ impl Concat {
     }
 }
 
-/// Scatter is equivelent to:
+/// Scatter is equivalent to:
 /// result = torch.zeros(shape)
 /// result[index] = node.evaluate()
 /// but index is considered dimwise
@@ -1442,6 +1443,7 @@ impl Scatter {
         shape: Shape,
         name: Option<Name>,
     ) -> Result<Self> {
+        let index = index.canonicalize(&shape);
         let index_shape = index.apply_to_shape(&shape);
         if index.all_uslices().is_none() {
             return Err(ConstructError::ScatterIndexTypeUnimplemented { index }.into());
