@@ -12,9 +12,7 @@ use rr_util::{
     cached_method,
     caching::FastUnboundedCache,
     eq_by_big_hash::EqByBigHash,
-    impl_eq_by_big_hash,
-    name::Name,
-    pycall, python_error_exception,
+    impl_eq_by_big_hash, pycall, python_error_exception,
     rearrange_spec::{ExpandToSpecOrShape, OpShape, OpSize, RearrangeSpec},
     sv,
     symbolic_size::SymbolicSizeProduct,
@@ -344,6 +342,7 @@ pub fn expand_node(
 
             let inputs = if inputs.iter().all(|x| x.ndim() == inputs[0].ndim()) {
                 // if statment just so that if we're going to fail anyway, we skip this case
+                // (and we don't have to handle this)
                 let new_shape: OpShape = (0..inputs[0].ndim())
                     .map(|i| {
                         if i == new_axis {
@@ -580,6 +579,7 @@ pub fn expand_node(
             // [          n_2, n_3,           b_2, y_0]
             // [n_0, n_1, n_2, n_3, b_0, b_1, b_2, z_0, z_1]
             // (n is for 'new')
+            // (we first need to make sure that b0, ..., b_2 match up by resolving symbolic sizes)
             //
             // Then we'll first move new dims into the symbol part as needed to
             // make batching 'contiguous' (via a rearrange)
@@ -603,18 +603,63 @@ pub fn expand_node(
                 .collect();
             let current_batch_rank = *current_node_batch_ranks.iter().max().unwrap();
 
-            let new_node_batch_ranks: Vec<_> = new_nodes
-                .iter()
-                .zip(&node.nodes)
-                .map(|(node, orig_node)| node.ndim().checked_sub(orig_node.ndim()).unwrap())
-                .collect();
-
             let spec_circuit_batch_rank = batch_ranks[0]; // spec circuit is 0th child
+            let new_node_batch_over_ranks: Vec<_> = batch_ranks[1..].to_vec(); // and then nodes are rest
+
+            // set symbolic shapes for b0, ..., b_n (see above)
+            let expanded_old_batch_shape: Shape = (0..current_batch_rank)
+                .map(|i| {
+                    find_non_symbolic(
+                        &new_nodes
+                            .iter()
+                            .zip(&current_node_batch_ranks)
+                            .zip(&new_node_batch_over_ranks)
+                            .filter_map(|((x, &current_this_node_batch_rank), &batch_over_rank)| {
+                                let len_from_end = current_batch_rank - i;
+                                current_this_node_batch_rank
+                                    .checked_sub(len_from_end)
+                                    .map(|j| x.shape()[batch_over_rank + j])
+                            })
+                            .collect::<Vec<_>>(),
+                    )
+                })
+                .collect();
+            let new_nodes = new_nodes
+                .into_iter()
+                .zip(&current_node_batch_ranks)
+                .zip(&new_node_batch_over_ranks)
+                .map(|((inp, &current_this_node_batch_rank), &batch_over_rank)| {
+                    let batch_dims_skipped = current_batch_rank - current_this_node_batch_rank;
+                    let new_inp_shape: OpShape = iter::repeat(None)
+                        .take(batch_over_rank)
+                        .chain(
+                            expanded_old_batch_shape[batch_dims_skipped..]
+                                .iter()
+                                .map(Some),
+                        )
+                        .chain(
+                            iter::repeat(None).take(
+                                inp.rank()
+                                    .checked_sub(batch_over_rank + current_this_node_batch_rank)
+                                    .unwrap(),
+                            ),
+                        )
+                        .map(|x| x.cloned().into())
+                        .collect();
+                    SetSymbolicShape::some_set_and_symbolic_neq(inp.clone(), new_inp_shape, None)
+                })
+                .collect::<Result<Vec<_>>>()
+                .with_context(|| {
+                    format!(
+                        "failed to set symbolic in expand module, old module={:?}",
+                        node
+                    )
+                })?;
 
             // how many dims do we have to move to make batching 'contiguous'
             let rank_to_push_into_spec_circuit = current_node_batch_ranks
                 .iter()
-                .zip(&new_node_batch_ranks)
+                .zip(&new_node_batch_over_ranks)
                 .map(|(this_current_batch_rank, this_new_batch_rank)| {
                     if *this_current_batch_rank == current_batch_rank {
                         0
@@ -631,14 +676,14 @@ pub fn expand_node(
                     .spec
                     .arg_specs
                     .iter()
-                    .zip(new_node_batch_ranks)
+                    .zip(new_node_batch_over_ranks)
                     .zip(new_nodes)
                     .zip(current_node_batch_ranks)
                     .map(
                         |(((arg_spec, this_new_batch_rank), node), this_current_batch_rank)| {
                             let this_sym_extra_rank =
                                 rank_to_push_into_spec_circuit.min(this_new_batch_rank);
-                            // move dims to make batching contiguous
+                            // move some of new batch rank dims to make batching contiguous
                             let new_sym = Symbol::new(
                                 node.shape()[this_new_batch_rank - this_sym_extra_rank
                                     ..this_new_batch_rank]
@@ -658,8 +703,12 @@ pub fn expand_node(
                                     .name
                                     .map(|x| format!("{} shift_batch", x).into());
                                 let spec = RearrangeSpec::new_permute(
-                                    (this_sym_extra_rank..up_to_old_sym_rank)
-                                        .chain(0..this_sym_extra_rank)
+                                    (0..this_new_batch_rank - this_sym_extra_rank)
+                                        .chain(this_new_batch_rank..up_to_old_sym_rank)
+                                        .chain(
+                                            this_new_batch_rank - this_sym_extra_rank
+                                                ..this_new_batch_rank,
+                                        )
                                         .chain(up_to_old_sym_rank..node_rank)
                                         .collect(),
                                 )
