@@ -264,7 +264,7 @@ impl Parser {
                 format!(r"(?:{}\s*:\s*)?{}\s*", AXIS_NAME_MATCH, *SIZE_PROD_MATCH);
             let trailing_comment_match = r"(?:#.*?)?"; // TODO: test this never matches stuff it's not supposed to...
             let full_regex = format!(
-                r"^({ws})(?:(\d+)|{nm})(?: \s*{nm})?( \s*AD)?(?: \s*\[((?:{sh},\s*)*(?:{sh})?)\])?(?: \s*([a-zA-Z]+))?(?: \s*(.*?))?(?:{ws}){com}$",
+                r"^({ws})(?:(\d+)|{nm})(?: \s*{nm})?( \s*AD)?(?: \s*\[\s*((?:{sh},\s*)*(?:{sh})?)\])?(?: \s*([a-zA-Z]+))?(?: \s*(.*?))?(?:{ws}){com}$",
                 ws = WHITESPACE,
                 nm = name_match_capture,
                 sh = shape_axis_match,
