@@ -1,4 +1,7 @@
-use std::fmt::{self, Display};
+use std::{
+    collections::hash_map::Entry,
+    fmt::{self, Display},
+};
 
 use anyhow::{anyhow, bail, Context, Result};
 use circuit_base::{
@@ -23,7 +26,7 @@ use rr_util::{
     },
     pycall, python_error_exception, sv,
     tensor_util::{Shape, TorchDevice, TorchDeviceDtype},
-    timed,
+    timed, unwrap,
     util::{arc_ref_clone, with_context_failable, HashBytes},
 };
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
@@ -120,17 +123,33 @@ impl Schedule {
         Ok(())
     }
 
-    pub fn replace_tensors(&self, map: HashMap<HashBytes, Tensor>) -> PyResult<Self> {
+    #[args(allow_missing = "false")]
+    pub fn replace_tensors(
+        &self,
+        map: HashMap<HashBytes, Tensor>,
+        allow_missing: bool,
+    ) -> Result<Self> {
         let mut result = self.clone();
         for (k, v) in map {
             if !result.old_constant_hashes.contains_key(&k) {
+                if allow_missing {
+                    continue;
+                }
                 return Err(PyErr::new::<pyo3::exceptions::PyKeyError, _>(
                     "key circuit wasn't present in original",
-                ));
+                )
+                .into());
             }
-            result
-                .constants
-                .insert(result.old_constant_hashes[&k], Array::new(v, None).into());
+            // should be present because we checked above
+            let entry = unwrap!(
+                result.constants.entry(result.old_constant_hashes[&k]),
+                Entry::Occupied
+            );
+            if entry.get().shape() != v.shape() {
+                bail!("tensor shape doesn't match entry shape!");
+            }
+
+            *entry.into_mut() = Array::new(v, None).into();
         }
         Ok(result)
     }
@@ -149,16 +168,16 @@ impl Schedule {
         Ok(result)
     }
     #[args(settings = "Default::default()")]
-    pub fn evaluate(&self, settings: OptimizationSettings) -> Tensor {
-        let eval = || {
+    pub fn evaluate(&self, settings: OptimizationSettings) -> Result<Tensor> {
+        let eval = |timed| -> Result<_> {
             let result = if settings.adjust_numerical_scale {
-                evaluate_schedule_adjust_numerical_scale(self, settings)
+                evaluate_schedule_adjust_numerical_scale(self, settings)?
                     [&self.output_circuit.as_ref().unwrap().0]
                     .clone()
             } else {
-                evaluate_schedule(self)[&self.output_circuit.as_ref().unwrap().0].clone()
+                evaluate_schedule(self)?[&self.output_circuit.as_ref().unwrap().0].clone()
             };
-            if self.device_dtype.device != TorchDevice::Cpu {
+            if timed && self.device_dtype.device != TorchDevice::Cpu {
                 Python::with_gil(|py| {
                     PY_UTILS
                         .torch
@@ -170,15 +189,31 @@ impl Schedule {
                         .unwrap()
                 });
             }
-            result
+            Ok(result)
         };
-        timed!(eval(), 10, settings.verbose >= 1)
+        let timed = settings.verbose >= 1;
+        Ok(timed!(eval(timed)?, 10, timed))
+    }
+
+    // TODO: I think we can still panic on other non-explicitly computable stuff? (e.g., cumulants etc)
+    pub fn check_no_syms(&self) -> Result<()> {
+        let mut iter = self
+            .constants
+            .iter()
+            .filter_map(|x| x.1.as_symbol())
+            .peekable();
+        if iter.peek().is_some() {
+            bail!(SchedulingError::EvaluateCalledWithSyms {
+                syms: iter.cloned().collect()
+            })
+        }
+        Ok(())
     }
 
     #[args(settings = "Default::default()")]
-    pub fn evaluate_many(&self, settings: OptimizationSettings) -> Vec<Tensor> {
-        let single = self.evaluate(settings);
-        self.split(single)
+    pub fn evaluate_many(&self, settings: OptimizationSettings) -> Result<Vec<Tensor>> {
+        let single = self.evaluate(settings)?;
+        Ok(self.split(single))
     }
 
     pub fn split(&self, tensor: Tensor) -> Vec<Tensor> {
@@ -364,6 +399,9 @@ pub enum SchedulingOOMError {
 pub enum SchedulingError {
     #[error("circuit={circuit:?} ({e_name})")]
     NotExplicitlyComputable { circuit: CircuitRc },
+
+    #[error("syms={syms:?} ({e_name})")]
+    EvaluateCalledWithSyms { syms: Vec<Symbol> },
 }
 
 impl Display for ScheduleStats {
@@ -428,7 +466,8 @@ where
     })
 }
 
-pub fn evaluate_schedule(schedule: &Schedule) -> HashMap<usize, Tensor> {
+pub fn evaluate_schedule(schedule: &Schedule) -> Result<HashMap<usize, Tensor>> {
+    schedule.check_no_syms()?;
     let mut live: HashMap<usize, Tensor> = schedule
         .constants
         .iter()
@@ -460,7 +499,12 @@ pub fn evaluate_schedule(schedule: &Schedule) -> HashMap<usize, Tensor> {
                     circ.print().unwrap()
                 }
                 let result = result_err.unwrap();
-                assert!(result.shape()[..] == circ.info().shape[..]);
+                assert!(
+                    result.shape() == circ.shape(),
+                    "circ={:?}, result_shape={:?}",
+                    circ,
+                    result.shape()
+                );
                 live.insert(*key, result);
             }
             Instruction::Drop(key) => {
@@ -469,7 +513,7 @@ pub fn evaluate_schedule(schedule: &Schedule) -> HashMap<usize, Tensor> {
         };
         unsafe { with_gil_pool(run) }
     }
-    live
+    Ok(live)
 }
 
 /// evaluate a circuit while measuring the numerical scale of tensor contents
@@ -477,7 +521,9 @@ pub fn evaluate_schedule(schedule: &Schedule) -> HashMap<usize, Tensor> {
 pub fn evaluate_schedule_adjust_numerical_scale(
     schedule: &Schedule,
     settings: OptimizationSettings,
-) -> HashMap<usize, Tensor> {
+) -> Result<HashMap<usize, Tensor>> {
+    schedule.check_no_syms()?;
+
     // we store (tensor, scale) where scale is a number the tensor's been multiplied by
     // so (tensor(1e10),1.0) evaluates to same as (tensor(1),1e-10)
     let mul = |tup: &(Tensor, f64), m: f64| -> (Tensor, f64) {
@@ -621,9 +667,11 @@ pub fn evaluate_schedule_adjust_numerical_scale(
         };
         unsafe { with_gil_pool(run) }
     }
-    live.iter()
+    let out = live
+        .iter()
         .map(|(k, v)| (*k, set_scale(v, 1.0).0))
-        .collect()
+        .collect();
+    Ok(out)
 }
 
 /// this supports dropping and recomputing. if you have an Evaluate(CircuitRc) of something you already evaluated
@@ -927,7 +975,7 @@ pub fn scheduled_evaluate(circuit: CircuitRc, settings: OptimizationSettings) ->
             &mut OptimizationContext::new_settings(settings),
         )
     }?;
-    Ok(evaluate_schedule(&schedule)[&schedule.output_circuit.unwrap().0].clone())
+    Ok(evaluate_schedule(&schedule)?[&schedule.output_circuit.unwrap().0].clone())
 }
 
 fn schedule_module_spec(spec: &ModuleSpec, context: &mut OptimizationContext) -> Schedule {
