@@ -72,6 +72,8 @@ def log_exp_p_1_fn(x: torch.Tensor):
 
 
 generalfunctions = {
+    "sin": torch.sin,
+    "cos": torch.cos,
     "gelu": torch.nn.functional.gelu,
     "relu": torch.relu,
     "step": lambda x: torch.where(
@@ -96,6 +98,8 @@ generalfunctions = {
     "logit": torch.logit,
 }
 
+pow = torch.pow
+
 
 def check_canon_idx(i: int, count: int):
     assert count >= 0, count
