@@ -79,6 +79,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         circuit_base::generalfunction::get_shape_info_simple,
         m
     )?)?;
+    m.add_function(wrap_pyfunction!(circuit_base::generalfunction::pow, m)?)?;
     circuit_base::generalfunction::register(py, m)?;
     m.add_class::<circuit_base::generalfunction::GeneralFunctionIndexSpec>()?;
     m.add_class::<circuit_base::generalfunction::GeneralFunctionSpecTester>()?;
@@ -244,7 +245,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         m
     )?)?;
     m.add_function(wrap_pyfunction!(
-        circuit_rewrites::compiler_strip::strip_names_py,
+        circuit_rewrites::compiler_strip::strip_names_and_tags_py,
         m
     )?)?;
     m.add_function(wrap_pyfunction!(deep_maybe_distribute_py, m)?)?;
